#!/usr/bin/perl

#created by Matthew Hestand, Michiel van Galen, Michel Villerius

#Copyright 2010 Leiden University Medical Center http://www.lgtc.nl/GAPSS


#This file is part of GAPSS.
#
#    GAPSS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GAPSS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GAPSS.  If not, see <http://www.gnu.org/licenses/>.

#===============================================#
#	Bowtie2IntermediateGapssFormat		#
#===============================================#
#       Created by 	m.van_galen@lumc	#
#       		August, 2009		#
#===============================================#

## Intermediate Gapss Format (IGF) : Custom designed format which any mapping output can be converted to. Parsable by GAPSS
## IGF: <CHR>	<START>	<END>	<STRAND>	<SEQ>	<QUAL>	<MM>	<MULTPLE>

use strict;

#======================
# Check input parameters
#======================
if (@ARGV < 4){
	if (($ARGV[0] eq "--help") or (!$ARGV[0])){
		print "\nUSAGE: $0 -i <STRING> -o <STRING>
		
	Options:
	-i	REQ	Filename containing Bowtie output
	-o	REQ	Desired name for IGF output file
\n";	
		exit(1);		
	}
	else{
		print "\nFor help on how to run this tool:\n\n$0 --help\n\n";
		exit(1);
	}
}
my %params;
my %seen_args;
my %used_args;
foreach my $i (0..$#ARGV){
	$seen_args{$ARGV[$i]}++;
	if($ARGV[$i] eq "-i"){
		if($ARGV[$i+1]){
			$params{"in_file"}=$ARGV[$i+1];
			$used_args{$ARGV[$i]}++;
			$used_args{$ARGV[$i+1]}++;			
		}
		else{
			print STDERR "Please enter valid input filename ...\n";
			exit(2);     
		}
	}
	elsif($ARGV[$i] eq "-o"){
		if($ARGV[$i+1]){
			$params{"out_file"}=$ARGV[$i+1];
			$used_args{$ARGV[$i]}++;
			$used_args{$ARGV[$i+1]}++;			
		}else{
			print STDERR "Please enter valid output filename ...\n";
			exit(2);     
		}
	}	
}
# Check for necesary arguments and existence of files supplied
foreach my $seen_this_arg(keys %seen_args){if(!exists $used_args{$seen_this_arg}){die("Error, encountered unused/invalid argument $seen_this_arg\n");}}
if(!exists($params{"in_file"})){die("No input file submitted ...\n");}
if(!-e $params{"in_file"}){die("Input file ".$params{"in_file"}." not found ...\n")}
if(!exists($params{"out_file"})){die("No output file submitted ...\n");}
if($params{"out_file"} eq $params{"in_file"}){die ("Inputfile can't be the same as output file ...\n");}

#======================
# RUN
#======================
&check_input($params{"in_file"});
&parse_bowtie($params{"in_file"},$params{"out_file"});

#======================
# SUBS
#======================
# Check input on bowtie format
sub check_input{
#======================
	my $in_file = shift(@_); 
	my $check_this_line = `head -1 $in_file`; 									# Head the file 
	my @frags = split(/\t/, $check_this_line); 									# Cut it up
	my $read_len = length($frags[4]);
	unless($frags[4] =~ m/[ACGTN]{$read_len}/i){die("Read doesn't look like sequence...\n");} 			# Check if the read consists only of allowed characters ACTGN case insensitive for the whole length
	unless($frags[3] !~ m/\D+/){die("Position not numeric...\n");} 							# Check for digits
	unless($frags[1] =~ m/^[+-]{1}$/){die("Expecting + or - for strand...\n");} 					# Check for plus or minus as strand
  	unless(length($frags[4]) == length($frags[5])){die("Qual and read are not of equal length...\n");} 		# Check if length is equal for the read and the qual string
}

#======================
sub parse_bowtie{
# FROM: header	+	chr17	5877937	GTGTGTGTGTGTGTGTGTGTGTGTGTGTGTGTGTG	IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII	52544	
# TO: chr17	5877938	5877972	+	GTGTGTGTGTGTGTGTGTGTGTGTGTGTGTGTGTG	IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII	
#======================
	my $in_file = shift(@_); # Parse and open files
	my $out_file = shift(@_);
	open(IN, "$in_file");
	open(OUT, ">$out_file");
	while(<IN>){ # Cycle through the file
		chop $_;
		my @frags = split(/\t/, $_);
		my $end = $frags[3]; # Save the end as start and add the lenght in the next line
		$end += length($frags[4]); # Because the zero based offset we can just add the length here. If offset changes, substract 1
		my $start = $frags[3]; # Start is just the start +1 in next line
		$start++; # Because of the zero based offset for bowtie we add one to the start here
		print OUT $frags[2]."\t".$start."\t".$end."\t".$frags[1]."\t".$frags[4]."\t".$frags[5]."\t".$frags[7]."\t".$frags[6]."\n"; # Print the parts of the line tab sep'ed
		
	}
	close(IN); # Cleanup ...
	close(OUT);
}

