#!/usr/bin/perl

#created by Matthew Hestand, Michiel van Galen, Michel Villerius

#Copyright 2010 Leiden University Medical Center http://www.lgtc.nl/GAPSS


#This file is part of GAPSS.
#
#    GAPSS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GAPSS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GAPSS.  If not, see <http://www.gnu.org/licenses/>.

#run as: perl GAPSS_compress_regions.pl "region file"

#################################################################################

#works on files that include peak regions and the number tags in region
#example line: Chr:start-stop	region_length	#_tags_in_regions	area_under_curve	peak_height	peak_location(s)
#example line: chr17:5877938-5877972	35	4	107	4	 chr17:5877938-5877939
#example line: chr17:5887938-5887939	2	1	2	1	 chr17:5887938-5887939
#example line: chr17:11656033-11656067	35	2	70	2	 chr17:11656033-11656067



#define files
$region_file = $ARGV[0];# the GAPSS region file to compress
$length_to_combine = $ARGV[1];# a number
$outfile_name = $region_file;
$outfile_name =~ s/\.region$//;
$outfile_name = $outfile_name.".compressed_region";

#check input file format
my $first_lines = `head -n 2 $region_file`;
my @first_lines = split(/\n/,$first_lines);
#if (@first_lines[1] =~ /^chr[0-9]*:[0-9]*\-[0-9]*\t[0-9]*\t[0-9]*\t[0-9]*\t[0-9]*\tchr[0-9]*:[0-9]*\-[0-9]*/){}else{die "file $file_to_convert is not in correct format, should be GAPSS universal alignment output format\n";}

#############################################################################################
#open region file
open (region_file, $region_file) || die "can't open file $region_file\n";

my %hash_names;

while (<region_file>){
    chop;
	#save file into seperate hashes per chromosome
	my @line_elements = split(/\:/,$_);
	$line_elements[0] =~ s/chr//;
	my $chr = $line_elements[0];
	if ($chr =~ /\#/){}else{
		my @line_elements2 = split(/\-/,$line_elements[1]);
		$$chr{$line_elements2[0]} = $_;
		$hash_names{$chr}="hit";
	}
}
close (region_file)|| die "Could not close $region_file\n";

#############################################################################################
#sort hashes and look for nearby regions, and write to outfile
open (out_file, ">$outfile_name") || die "could not open outfile called $outfile_name\n";
print out_file "Chr#:start-stop\tregion_length\t#_tags_in_regions\tarea_under_curve\tpeak_height\tpeak_location(s)\n";
my @hash_names = keys %hash_names;
@hash_names = sort { $a <=> $b } @hash_names;
my @blank;
my @combine;

foreach my $chromosome (@hash_names) {
	my @starts = keys %$chromosome;
	@starts = sort { $a <=> $b } @starts;
	my $number_regions_in_chr = $#starts;
	my $current_region = 1;

	#set for first element
		my $start = $starts[0];my $start_old = $start;
		my @elements_line = split(/\t/,$$chromosome{$start});
		my @start_stop = split(/\-/,$elements_line[0]);
		my $end_old = $start_stop[1];
		my $length_line_hits_old = $elements_line[1];
		my $total_tags_in_region_old = $elements_line[2];
		my $total_tags_old =$elements_line[3];
		my $peak_old = $elements_line[4];
		my $peak_region_old = $elements_line[5];



	until ($current_region>$number_regions_in_chr){
		my $start = $starts[$current_region];
		my @elements_line = split(/\t/,$$chromosome{$start});
		my @start_stop = split(/\-/,$elements_line[0]);
		my $end = $start_stop[1];
		my $length_line_hits = $elements_line[1];
		my $total_tags_in_region = $elements_line[2];
		my $total_tags =$elements_line[3];
		my $peak = $elements_line[4];
		my $peak_region = $elements_line[5];

		if (($start - $end_old - 1)<$length_to_combine) {
			#combine
			$end_old = $end;
			$total_tags_old = $total_tags_old+$total_tags;
			$total_tags_in_region_old = $total_tags_in_region+$total_tags_in_region_old;
			$length_line_hits_old = $length_line_hits_old + $length_line_hits;
			my $peaks_for_testing_string = $peak_old;
			if ($peak > $peaks_for_testing_string){$peak_old = $peak};
			if ($peak > $peaks_for_testing_string){$peak_region_old = $peak_region};
			if ($peak == $peaks_for_testing_string){$peak_region_old = $peak_region_old.",$peak_region"};
		}else{
			#else print previous
			my $length_to_print = $end_old - $start_old + 1;
			print out_file "chr$chromosome:$start_old-$end_old\t$length_to_print\t$total_tags_in_region_old\t$total_tags_old\t$peak_old\t$peak_region_old\n";
			$start_old = $start;
			$end_old = $end;
			$length_line_hits_old = $length_line_hits;
			$total_tags_old = $total_tags;
			$total_tags_in_region_old = $total_tags_in_region;
			$peak_old = $peak;
			$peak_region_old = $peak_region;
	
		}
		$current_region++;
	}
	my $length_to_print = $end_old - $start_old + 1;
	print out_file "chr$chromosome:$start_old-$end_old\t$length_to_print\t$total_tags_in_region_old\t$total_tags_old\t$peak_old\t$peak_region_old\n";
}

close (out_file) || die "Could not close outfile called $outfile_name\n";

