#!/usr/bin/perl

#created by Matthew Hestand, Michiel van Galen, Michel Villerius

#Copyright 2010 Leiden University Medical Center http://www.lgtc.nl/GAPSS


#This file is part of GAPSS.
#
#    GAPSS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GAPSS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GAPSS.  If not, see <http://www.gnu.org/licenses/>.

#use as: perl GAPSS_edit_linkers.pl "FASTQ file" "5 or 3" "linker sequence" "mm permitted in linker" "Y or N to overwrite previous file";

#edits a FASTQ file for linker sequences either at the 5' or 3' end with 0 or 1 mismatches

my $file_to_convert = $ARGV[0];#should be a fastq file
my $linker_5_or_3_prime = $ARGV[1];# 5 or 3
my $linker_sequence = $ARGV[2];# a sequence
my $number_missmatches_to_allow_in_linker= $ARGV[3];# 0 or 1
my $to_rewrite_over_file = $ARGV[4];#Y or N

#make sure enough arguments
if (@ARGV != 5){die "did not give proper input, should be: perl GAPSS_edit_linkers.pl \"FASTQ file\" \"5 or 3\" \"linker sequence\" \"mm permitted in linker\" \"Y or N to overwrite previous file\"\n";}

my $outfile = $file_to_convert.".edited_for_linkers";

my %results_of_linker_editing;


open(infile,$file_to_convert)||die "could not open file $file_to_convert to edit for linkers\n";
open(fastq_file2,">$outfile")||die "could not open file $outfile\n";
	my $length_cut=0;
	my $is_seq = "yes";
	while (<infile>){
		chop;
		$_ =~ s/\n//;
		if ($_ =~ /^\@/){
			print fastq_file2 "$_\n";
			$is_seq = "yes";
		}elsif ($_ =~ /^\+/){
			print fastq_file2 "$_\n";
			$is_seq = "no";
		}else{
		   if ($is_seq eq "yes"){
			my $cut_sequence=&edit_for_linker($_,$linker_sequence);	
			$length_cut=(length $_)-(length $cut_sequence);
			print fastq_file2 "$cut_sequence\n";
			#print "$cut_sequence length $length_cut\n";
			$results_of_linker_editing{$length_cut}=$results_of_linker_editing{$length_cut}+1;
		   }elsif ($is_seq eq "no"){
			my $quality_line_to_cut = $_;
			if ($linker_5_or_3_prime eq 5){$quality_line_to_cut = reverse($_);}
			until ($length_cut<1){
				chop $quality_line_to_cut;
				$length_cut=$length_cut-1;
			}
			if ($linker_5_or_3_prime eq 5){$quality_line_to_cut = reverse($quality_line_to_cut);;}
			print fastq_file2 "$quality_line_to_cut\n";
		   }else{
			die "was error in stript, variable \$is_seq\n";
		   }
		}
}
close(infile)||die "could not close file $file_to_convert to edit for linkers\n";
close(fastq_file2)||die "could not close file $outfile\n";

####print cutting results
my @keys_results = keys %results_of_linker_editing;
@keys_results = sort { $a <=> $b } @keys_results;
print "Results from linker editing:\n";
foreach my $key_result (@keys_results){
	print "\tlinker length $key_result was found $results_of_linker_editing{$key_result} times\n";

}

#write new file over old if requested, default is YES
if ($to_rewrite_over_file eq "Y"){
	system("mv $outfile $file_to_convert");
}

########################################################################################################################################################
sub edit_for_linker{
 my $seq_to_edit = $_[0];
 my $linker_sequence_sub = $_[1];
 if ($number_missmatches_to_allow_in_linker eq "0"){
	my $linker_length_sub = length $linker_sequence_sub;
	my $exit_while = "no";
	while (($linker_length_sub>2)&&($exit_while eq "no")){
		if( ($linker_5_or_3_prime eq "3")&&($seq_to_edit =~ /$linker_sequence_sub$/) ){
				$seq_to_edit =~ s/$linker_sequence_sub$//;
				$exit_while = "yes";
			
		}elsif( ($linker_5_or_3_prime eq "5")&&($seq_to_edit =~ /^$linker_sequence_sub/) ){
				$seq_to_edit =~ s/^$linker_sequence_sub//;
				$exit_while = "yes";
			
		}else{
			if($linker_5_or_3_prime eq "3"){chop($linker_sequence_sub);}elsif($linker_5_or_3_prime eq "5"){$linker_sequence_sub=reverse($linker_sequence_sub);chop($linker_sequence_sub);$linker_sequence_sub=reverse($linker_sequence_sub);}else{die "linker_5_or_3_prime is $linker_5_or_3_prime should be 5 or 3\n";}
			$linker_length_sub = length $linker_sequence_sub;
		}
	}
 }elsif ($number_missmatches_to_allow_in_linker eq "1"){
	my $linker_length_sub = length $linker_sequence_sub;
	my $exit_while = "no";
	while (($linker_length_sub>2)&&($exit_while eq "no")){
		#check perfect 3' match
		if( ($linker_5_or_3_prime eq "3")&&($seq_to_edit =~ /$linker_sequence_sub$/) ){
				$seq_to_edit =~ s/$linker_sequence_sub$//;
				$exit_while = "yes";
		#check perfect 5' match		
		}elsif( ($linker_5_or_3_prime eq "5")&&($seq_to_edit =~ /^$linker_sequence_sub/) ){
				$seq_to_edit =~ s/^$linker_sequence_sub//;
				$exit_while = "yes";
		#check mismatch
		}else{
			#make all possible reg exp for this length linker
			my @linker_possibilities_reg_exp;
			my $counter_length = $linker_length;
			until ($counter_length<1){
				my $temp_linker_sequence_sub = $linker_sequence_sub;
				my $before = $temp_linker_sequence_sub;$before =~ s/[CATGcatgNn]{$counter_length}$//;
				my $after_counter_length=$linker_length-$counter_length+1;
				my $after = $temp_linker_sequence_sub;$after =~ s/^[CATGcatgNn]{$after_counter_length}//;
				push (@linker_possibilities_reg_exp, "$before\[A-Za-z\]\{1\}$after");
				$counter_length=$counter_length-1;
			}
			#print "original: $linker_sequence_sub\n";
			#foreach $debugcrap (@linker_possibilities_reg_exp){print "$debugcrap\n";}
			#die "debug\n";

			#check all reg exp
			if($linker_5_or_3_prime eq "3"){
				foreach $my_reg_to_check (@linker_possibilities_reg_exp){
					if (($seq_to_edit =~ /$my_reg_to_check$/)&&($exit_while eq "no")){
						$seq_to_edit =~ s/$my_reg_to_check$//;
						$exit_while = "yes";
					}
				}
			}elsif($linker_5_or_3_prime eq "5"){
				foreach $my_reg_to_check (@linker_possibilities_reg_exp){
					if (($seq_to_edit =~ /^$my_reg_to_check/)&&($exit_while eq "no")){
						$seq_to_edit =~ s/^$my_reg_to_check//;
						$exit_while = "yes";
					}
				}
			}

			if ($exit_while eq "no"){
				if($linker_5_or_3_prime eq "3"){chop($linker_sequence_sub);}elsif($linker_5_or_3_prime eq "5"){$linker_sequence_sub=reverse($linker_sequence_sub);chop($linker_sequence_sub);$linker_sequence_sub=reverse($linker_sequence_sub);}else{die "linker_5_or_3_prime is $linker_5_or_3_prime should be 5 or 3\n";}
				$linker_length_sub = length $linker_sequence_sub;
			}
		}
	}


 }else{
	die "number_missmatches_to_allow_in_linker was $number_missmatches_to_allow_in_linker, should be 0 or 1\n";
 }

return $seq_to_edit;
}
########################################################################################################################################################



