#!/usr/bin/perl

#created by Matthew Hestand, Michiel van Galen, Michel Villerius

#Copyright 2010 Leiden University Medical Center http://www.lgtc.nl/GAPSS


#This file is part of GAPSS.
#
#    GAPSS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GAPSS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GAPSS.  If not, see <http://www.gnu.org/licenses/>.

#run as: perl GAPSS_make_regions.pl "general alignment format file" "split by strand Y or N"

#input is a GAPSS edited alignment file
#output is in format:
#		chr:start-stop	region_length	#tags_in_region	#area_under_curve	#peak_location	#peak_height

my $file_to_convert = $ARGV[0];# format is: chr17	5877938	5877972	+	GTGTGTGTGTGTGTGTGTGTGTGTGTGTGTGTGTG	IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
my $split_by_strand = $ARGV[1];# Y or N


my $outfile = $file_to_convert.".region";
my $outfileF = $file_to_convert."_F.region";
my $outfileR = $file_to_convert."_R.region";

#check file has correct format
my $first_line = `head -n 1 $file_to_convert`;
#if ($first_line =~ /^chr[0-9]*\t[0-9]*\t[0-9]*\t[\+\-]\t[TAGCtagcNn]*\t[A-Za-z0-9]*/){}else{die "file $file_to_convert is not in correct format, should be GAPSS universal alignment output format\n";}


#get all chr used
my $line_with_all_chr = `cut -f1 $file_to_convert | sort|uniq`;
my @list_of_chr = split(/\n/,$line_with_all_chr);

#open outfile to write output to
if ($split_by_strand eq "N"){
	open(outfile,">$outfile")||die"could not open output file for regions: file $outfile\n";
	#print header
	print outfile "Chr#:start-stop\tregion_length\t#_tags_in_regions\tarea_under_curve\tpeak_height\tpeak_location(s)\n";
}elsif ($split_by_strand eq "Y"){
	open(outfileF,">$outfileF")||die"could not open output file for regions: file $outfileF\n";
	#print header
	print outfileF "Chr#:start-stop (F_strand)\tregion_length\t#_tags_in_regions\tarea_under_curve\tpeak_height\tpeak_location(s)\n";
	open(outfileR,">$outfileR")||die"could not open output file for regions: file $outfileR\n";
	#print header
	print outfileR "Chr#:start-stop (R_strand)\tregion_length\t#_tags_in_regions\tarea_under_curve\tpeak_height\tpeak_location(s)\n";

}else{
	die "did not request stran split correct: was $split_by_strand\n";
}
#get regions, work per chr
foreach my $working_chr (@list_of_chr){

	#make temp file of working chr, by strand
	my $temp_chr_align_file="";my $temp_chr_align_file_F="";my $temp_chr_align_file_R="";
	if ($split_by_strand eq "N"){
		$temp_chr_align_file = "$file_to_convert"."$working_chr.temp_GAPSS_region_out666";
		system ("grep -P \"^$working_chr\\t\" $file_to_convert |sort -n  -n -k2 \> $temp_chr_align_file");
	}elsif ($split_by_strand eq "Y"){
		$temp_chr_align_file_F = "$file_to_convert".$working_chr."_F.temp_GAPSS_region_out666";
		$temp_chr_align_file_R = "$file_to_convert".$working_chr."_R.temp_GAPSS_region_out666";
		system ("grep -P \"^$working_chr\\t[0-9]*\\t[0-9]*\\t\\+\\t\" $file_to_convert |sort -n  -n -k2 \> $temp_chr_align_file_F");
		system ("grep -P \"^$working_chr\\t[0-9]*\\t[0-9]*\\t\\-\\t\" $file_to_convert |sort -n  -n -k2 \> $temp_chr_align_file_R");
	}else{
		die "did not request stran split correct: was $split_by_strand\n";
	}

	#make regions from temp file(s)
	if ($split_by_strand eq "N"){
		&make_regions_from_temp_file($temp_chr_align_file);
	}elsif ($split_by_strand eq "Y"){
		&make_regions_from_temp_file($temp_chr_align_file_F);
		&make_regions_from_temp_file($temp_chr_align_file_R);
	}else{
		die "did not request strand split correct: was $split_by_strand\n";
	}

	#remove temp chr file(s)
	if ($split_by_strand eq "N"){
		system("rm $temp_chr_align_file");
	}elsif ($split_by_strand eq "Y"){
		system("rm $temp_chr_align_file_F");
		system("rm $temp_chr_align_file_R");
	}else{
		die "did not request strand split correct: was $split_by_strand\n";
	}
	

}

#close outfile to write output to
if ($split_by_strand eq "N"){
close(outfile)||die"could not close output file for regions: file $outfile\n";
}elsif ($split_by_strand eq "Y"){
close(outfileF)||die"could not close output file for regions: file $outfileF\n";
close(outfileR)||die"could not close output file for regions: file $outfileR\n";
}else{
	die "did not request stran split correct: was $split_by_strand\n";
}
#########################################################################################################################################################
sub make_regions_from_temp_file{
	my $temp_file_in_sub_for_regions = $_[0];

	#open and read file into regions
	open(infile,$temp_file_in_sub_for_regions)||die"could not open temp file $temp_file_in_sub_for_regions to make regions\n";
	my $region_start = 0;my $region_stop = 0; my $region_total_tags = 0;my $area_under_the_curve=0;my %peak_creation_hash;
	my $current_start;my $current_stop;my $current_chr;
	while (<infile>){
		chop;
		$_ =~ s/\n//;
		my @align_line_elements = split(/\t/,$_);#chr start stop strand seq qual
		$current_start = @align_line_elements[1];
		$current_stop = @align_line_elements[2];
		$current_chr = @align_line_elements[0];

		#if overlaps previous sequence calculate region, otherwise print region
		if ( (($current_start<$region_start)&&($current_stop<$region_start)) || (($current_start>$region_stop)&&($current_stop>$region_stop)) ){
			#print region (except first time when everything is 0
			if ($region_start == 0){}else{ 
				my $region_length = $region_stop-$region_start+1;

				#get peak defined
				my @bps_of_region_for_peak_define = keys %peak_creation_hash;
				@bps_of_region_for_peak_define = sort { $a <=> $b } @bps_of_region_for_peak_define;
				my $peak_height=0;
				#get peak height
				foreach my $bp_get_height (@bps_of_region_for_peak_define){
					if ($peak_creation_hash{$bp_get_height}>$peak_height){$peak_height=$peak_creation_hash{$bp_get_height};}
				}
				#get peak region
				my $peak_regions;my $previous_peak_bp = -1;my $printed_last = "yes";
				foreach my $bp_get_region (@bps_of_region_for_peak_define){
					if ($peak_creation_hash{$bp_get_region}==$peak_height){
						if (($bp_get_region-$previous_peak_bp)==1){
							$previous_peak_bp=$bp_get_region;
							$printed_last =$bp_get_region;
						}else{
							$peak_regions=$peak_regions."$previous_peak_bp, $current_chr:$bp_get_region-";
							$previous_peak_bp=$bp_get_region;
						}
					}
				}
				if ($printed_last eq "yes"){
					$peak_regions=~s/, [0-9]*\-$//;
				}else{
					$peak_regions=$peak_regions."$printed_last";
				}
				$peak_regions=~s/^-1, //;

				#actual print line
				#print outfile "$current_chr:$region_start-$region_stop\t$region_length\t$region_total_tags\t$area_under_the_curve\t$peak_height\t$peak_regions\n"; 
				if ($split_by_strand eq "N"){
					print outfile "$current_chr:$region_start-$region_stop\t$region_length\t$region_total_tags\t$area_under_the_curve\t$peak_height\t$peak_regions\n";
 				}elsif ($split_by_strand eq "Y"){
					if ($temp_file_in_sub_for_regions =~ /_F.temp_GAPSS_region_out666/){
					print outfileF "$current_chr:$region_start-$region_stop\t$region_length\t$region_total_tags\t$area_under_the_curve\t$peak_height\t$peak_regions\n";
					}
					if ($temp_file_in_sub_for_regions =~ /_R.temp_GAPSS_region_out666/){
					print outfileR "$current_chr:$region_start-$region_stop\t$region_length\t$region_total_tags\t$area_under_the_curve\t$peak_height\t$peak_regions\n";
					}

				}
			}

			#reset variables with current data
			$region_start = $current_start;$region_stop = $current_stop;$region_total_tags = 1;$area_under_the_curve=$region_stop-$region_start+1;undef %peak_creation_hash;
			my $peak_in_counter = $current_start; 
			until ($peak_in_counter>$current_stop){
				if (exists($peak_creation_hash{$peak_in_counter})){$peak_creation_hash{$peak_in_counter}=$peak_creation_hash{$peak_in_counter}+1;}else{$peak_creation_hash{$peak_in_counter}=1;}
				$peak_in_counter++;
			}
		}else{
		    #work on region
			#define region start
			if ($current_start<$region_start){
				$region_start=$current_start#should not occur since data was sorted, but in case
			}
			#define region stop
			if ($current_stop>$region_stop){
				$region_stop=$current_stop;
			}	
			#define # tags in region
			$region_total_tags++;
			#define area under curve
			$area_under_the_curve = $area_under_the_curve+($region_stop-$region_start+1);
			#work on peak
			my $peak_in_counter = $current_start; 
			until ($peak_in_counter>$current_stop){
				if (exists($peak_creation_hash{$peak_in_counter})){$peak_creation_hash{$peak_in_counter}=$peak_creation_hash{$peak_in_counter}+1;}else{$peak_creation_hash{$peak_in_counter}=1;}
				$peak_in_counter++;
			}
		}
	}

	#if not printed and made into a region previously, print last line
		if ( (($current_start<$region_start)&&($current_stop<$region_start)) || (($current_start>$region_stop)&&($current_stop>$region_stop)) ){}else{
			#print region (except first time when everything is 0
			if ($region_start == 0){}else{ 
				my $region_length = $region_stop-$region_start+1;

				#get peak defined
				my @bps_of_region_for_peak_define = keys %peak_creation_hash;
				@bps_of_region_for_peak_define = sort { $a <=> $b } @bps_of_region_for_peak_define;
				my $peak_height=0;
				#get peak height
				foreach my $bp_get_height (@bps_of_region_for_peak_define){
					if ($peak_creation_hash{$bp_get_height}>$peak_height){$peak_height=$peak_creation_hash{$bp_get_height};}
				}
				#get peak region
				my $peak_regions;my $previous_peak_bp = -1;my $printed_last = "yes";
				foreach my $bp_get_region (@bps_of_region_for_peak_define){
					if ($peak_creation_hash{$bp_get_region}==$peak_height){
						if (($bp_get_region-$previous_peak_bp)==1){
							$previous_peak_bp=$bp_get_region;
							$printed_last =$bp_get_region;
						}else{
							$peak_regions=$peak_regions."$previous_peak_bp, $current_chr:$bp_get_region-";
							$previous_peak_bp=$bp_get_region;
						}
					}
				}
				if ($printed_last eq "yes"){
					$peak_regions=~s/, [0-9]*\-$//;
				}else{
					$peak_regions=$peak_regions."$printed_last";
				}
				$peak_regions=~s/^-1, //;

				#actual print line
				if ($split_by_strand eq "N"){
					print outfile "$current_chr:$region_start-$region_stop\t$region_length\t$region_total_tags\t$area_under_the_curve\t$peak_height\t$peak_regions\n";
 				}elsif ($split_by_strand eq "Y"){
					if ($temp_file_in_sub_for_regions =~ /_F.temp_GAPSS_region_out666/){
					print outfileF "$current_chr:$region_start-$region_stop\t$region_length\t$region_total_tags\t$area_under_the_curve\t$peak_height\t$peak_regions\n";
					}
					if ($temp_file_in_sub_for_regions =~ /_R.temp_GAPSS_region_out666/){
					print outfileR "$current_chr:$region_start-$region_stop\t$region_length\t$region_total_tags\t$area_under_the_curve\t$peak_height\t$peak_regions\n";
					}

				}
			}
		}

	#close file
	close(infile)||die"could not close temp file $temp_file_in_sub_for_regions to make regions\n";
}
