#!/usr/bin/perl

#created by Matthew Hestand, Michiel van Galen, Michel Villerius

#Copyright 2010 Leiden University Medical Center http://www.lgtc.nl/GAPSS


#This file is part of GAPSS.
#
#    GAPSS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GAPSS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GAPSS.  If not, see <http://www.gnu.org/licenses/>.

#run as: perl GAPSS_remove_replicates.pl  "FASTQ file" -write "Y or N or blank" -qual "value, otherwise use sanger default";

#this script runs on a FASTQ format file
#removes replicate lines, and sets quality scores to best possible
#by default re-writes over old file, otherwise rename "file"._no_replicates

#notes for user:
#	best sanger score = 
#	best Solexa score = 

#input: FASTQ file
#output: FASTQ file (original or new)

#set input
my $file_to_convert_sub;
my $value_for_quality_score;
my $to_rewrite_over_file;
#check arguments
##the next line does not work with 5 arguments given for now hash it out.
#if ((@ARGV == 0)||(@ARGV == 1)||(@ARGV == 3)){}else{die "incorrect arguments given\n";}
$file_to_convert_sub = $ARGV[0];
my $argument1 = $ARGV[1];#print "1 $argument1\n";
my $argument2 = $ARGV[2];#print "2 $argument2\n";
my $argument3 = $ARGV[3];#print "3 $argument3\n";
my $argument4 = $ARGV[4];#print "4 $argument4\n";
if (($argument1 eq "")||($argument1 eq "-write")||($argument1 eq "-qual")){}else{die "incorrect argument given\n";}
if (($argument3 eq "")||($argument3 eq "-write")||($argument3 eq "-qual")){}else{die "incorrect argument given\n";}
if ($argument1 eq "-write"){
	$to_rewrite_over_file=$argument2;
}elsif ($argument3 eq "-write"){
	$to_rewrite_over_file=$argument4;
}
if ($argument1 eq "-qual"){
	$value_for_quality_score=$argument2;
}elsif ($argument3 eq "-qual"){
	$value_for_quality_score=$argument4;
}


#set re-write to default if not specified
$to_rewrite_over_file =~ s/\n//;
if ($to_rewrite_over_file eq ""){
	$to_rewrite_over_file = "Y";
}elsif(($to_rewrite_over_file eq "Y")||($to_rewrite_over_file eq "N")){
}else{
	die "the request to write over file must be \"Y\" or \"N\" \n";
}
#set QC value to Sanger best () if no value given
$value_for_quality_score =~ s/\n//;
if ($value_for_quality_score eq ""){
	$value_for_quality_score="a";
}


#remove the replicates
	my $file_to_convert_sub_temp_out = $file_to_convert_sub."._no_replicates_temp666";
	my $file_to_convert_sub_temp_out2 = $file_to_convert_sub."._no_replicates";

	#system call to get only unique sequences
	system ("grep -P \"^@\" -A1 $file_to_convert_sub|grep -v -P \"\\-\\-\"|grep -v -P \"^@\"|sort|uniq > $file_to_convert_sub_temp_out");

	#convert sequences to FASTQ
	open (infile, $file_to_convert_sub_temp_out)||die "could not open file $file_to_convert_sub_temp_out\n";
	open (outfile, ">$file_to_convert_sub_temp_out2")||die "could not open file $file_to_convert_sub_temp_out2\n";
	while (<infile>){
		chop;
		$_ =~ s/\n//;
		my $quality_line = $_;
		$quality_line =~ s/[TAGCtagcNn]/$value_for_quality_score/g;
		print outfile "\@header\n$_\n+\n$quality_line\n";
	}
	close (infile)||die "could not close infile $file_to_convert_sub_temp_out\n";
	close (outfile)||die "could not close outfile $file_to_convert_sub_temp_out2\n";
	#clean out temp file
	system("rm $file_to_convert_sub_temp_out");

#write new file over old if requested, default is YES
if ($to_rewrite_over_file eq "Y"){
	system("mv $file_to_convert_sub_temp_out2 $file_to_convert_sub");
}

