#!/usr/bin/perl

#created by Matthew Hestand, Michiel van Galen, Michel Villerius

#Copyright 2010 Leiden University Medical Center http://www.lgtc.nl/GAPSS


#This file is part of GAPSS.
#
#    GAPSS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GAPSS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GAPSS.  If not, see <http://www.gnu.org/licenses/>.

#made Oct27 2009, Matt Hestand

#uses examples/info from Ensembl Variation API tutorial at:http://www.ensembl.org/info/docs/api/variation/variation_tutorial.html

################################################################
#input and set up

$ensembl_used = $ARGV[0];
#$ensembl_used = "human_Ens56";
#SNP file, first elements should be Chr \t position \t ref_nucleotide \t SNP_nucleotide \t ....
my $SNP_File = $ARGV[1];
my $outfile_name = $ARGV[2];




#ensembl Perl API locations
use lib '/usr/local/ensembl-api-56/ensembl/modules';
use lib '/usr/local/ensembl-api-56/ensembl-variation/modules';
use lib '/usr/local/ensembl-api-56/ensembl-compara/modules';
use DBI;
use Bio::EnsEMBL::Registry;
use Bio::EnsEMBL::Exon;
################################################################

#connect to Ensembl
my $registry = 'Bio::EnsEMBL::Registry';

$registry->load_registry_from_db(
    -host => 'ensembldb.ensembl.org',
    -user => 'anonymous'
);


my $slice_adaptor = $registry->get_adaptor('Homo_sapiens', 'core', 'slice');
my $vfa = $registry->get_adaptor('human', 'variation', 'variationfeature');

#####################################################################################################
#open SNP file
open(SNPfile,$SNP_File)||die "could not open SNP file $SNP_File\n";	

open(outfile,">$outfile_name")||die "could not open file $outfile_name\n";	
$header="Chrom\tPosition\tRef\tVar\tReads1\tReads2\tVarFreq\tStrands1\tStrands2\tQual1\tQual2\tPvalue\tKnown_SNP\tmatch_to_known_SNP\tESSENTIAL_SPLICE_SITE\tSTOP_GAINED\tSTOP_LOST\tFRAMESHIFT_CODING\tNON_SYNONYMOUS_CODING\tSPLICE_SITE\tSYNONYMOUS_CODING\tREGULATORY_REGION\tFIVE_PRIME_UTR\tTHREE_PRIME_UTR\tINTRONIC\tUPSTREAM\tDOWNSTREAM\tWITHIN_NON_CODING_GENE\tgenes_hit\tgene_symbols_hit\tdistance_to_coding_start\n";

print outfile "$header";

while (<SNPfile>){
  if ($_ =~ /^Chrom/){}else{
	chop;
	$_ =~ s/\n//;
	$_ =~ s/^chr//;

	#print output
  	print outfile "$_\t";

	#get chr start stop
	my @split_line_by_tabs = split(/\t/,$_);
	my $allele_reported = "@split_line_by_tabs[2]/@split_line_by_tabs[3]";


	my $chr = @split_line_by_tabs[0];
	my $start = @split_line_by_tabs[1];

	print "chr is $chr, start is $start\n";


	#get ID of known SNPs
	my $known_SNP_line = "";
	my $match_to_new_SNP = "no";
	my $slice = $slice_adaptor->fetch_by_region('chromosome',$chr,$start,$start); #get chromosome 25 in zebrafish
	my $vf_adaptor = $registry->get_adaptor('human', 'variation', 'variationfeature'); #get adaptor to VariationFeature object
	my $vfs = $vf_adaptor->fetch_all_by_Slice($slice); #return ALL variations defined in $slice
	foreach my $vf (@{$vfs}){
	  $name_known_SNP = $vf->variation_name;
	  $allele_known_SNP = $vf->allele_string;
	  $known_SNP_line = $known_SNP_line."$name_known_SNP($allele_known_SNP) ";
	  if ($allele_known_SNP eq $allele_reported){$match_to_new_SNP="yes";}
		#print "DEBUG known $allele_known_SNP $allele_reported\n";
	}




	#define new SNP types hits
	my $ESSENTIAL_SPLICE_SITE_hit = "";
	my $STOP_GAINED = "";
	my $STOP_LOST = "";
	my $FRAMESHIFT_CODING = "";
	my $NON_SYNONYMOUS_CODING = "";
	my $SPLICE_SITE = "";
	my $SYNONYMOUS_CODING = "";
	my $REGULATORY_REGION = "";
	my $FIVE_PRIME_UTR = "";
	my $THREE_PRIME_UTR = "";
	my $INTRONIC = "";
	my $UPSTREAM = "";
	my $DOWNSTREAM = "";
	my $WITHIN_NON_CODING_GENE = "";


	#get new SNP types: get the consequence types
	my $slice = $slice_adaptor->fetch_by_region('chromosome', $chr);
	my $new_vf = Bio::EnsEMBL::Variation::VariationFeature->new(
	  -start => $start,
	  -end => $start,
	  -slice => $slice,          
	  -allele_string => '$allele_reported',   
	  -strand => 1,
	  -map_weight => 1,
	  -adaptor => $vfa,          
	  -variation_name => 'newSNP',
	);
	my $cons = $new_vf->get_all_TranscriptVariations();
	foreach my $con(@{$new_vf->get_all_TranscriptVariations}) {
	  foreach my $string(@{$con->consequence_type}) {
		my $transcript_to_print = $con->transcript->stable_id;
		if ($string eq "ESSENTIAL_SPLICE_SITE_hit"){$ESSENTIAL_SPLICE_SITE_hit=$ESSENTIAL_SPLICE_SITE_hit."$transcript_to_print ";}
		if ($string eq "STOP_GAINED"){$STOP_GAINED=$STOP_GAINED."$transcript_to_print ";}
		if ($string eq "STOP_LOST"){$STOP_LOST=$STOP_LOST."$transcript_to_print ";}
		if ($string eq "FRAMESHIFT_CODING"){$FRAMESHIFT_CODING=$FRAMESHIFT_CODING."$transcript_to_print ";}
		if ($string eq "NON_SYNONYMOUS_CODING"){$NON_SYNONYMOUS_CODING=$NON_SYNONYMOUS_CODING."$transcript_to_print ";}
		if ($string eq "SPLICE_SITE"){$SPLICE_SITE=$SPLICE_SITE."$transcript_to_print ";}
		if ($string eq "SYNONYMOUS_CODING"){$SYNONYMOUS_CODING=$SYNONYMOUS_CODING."$transcript_to_print ";}
		if ($string eq "REGULATORY_REGION"){$REGULATORY_REGION=$REGULATORY_REGION."$transcript_to_print ";}
		if ($string eq "5PRIME_UTR"){$FIVE_PRIME_UTR=$FIVE_PRIME_UTR."$transcript_to_print ";}
		if ($string eq "3PRIME_UTR"){$THREE_PRIME_UTR=$THREE_PRIME_UTR."$transcript_to_print ";}
		if ($string eq "INTRONIC"){$INTRONIC=$INTRONIC."$transcript_to_print ";}
		if ($string eq "UPSTREAM"){$UPSTREAM=$UPSTREAM."$transcript_to_print ";}
		if ($string eq "DOWNSTREAM"){$DOWNSTREAM=$DOWNSTREAM."$transcript_to_print ";}
		if ($string eq "WITHIN_NON_CODING_GENE"){$WITHIN_NON_CODING_GENE=$WITHIN_NON_CODING_GENE."$transcript_to_print ";}
	  }
	}

	#for LOVD need Gene Symbol and position in coding (of gene)
	my $slice = $slice_adaptor->fetch_by_region( 'chromosome', $chr, $start, $start);
	#check slice for genes
	my $gene_line;my $gene_symbol_line;my $distance_to_coding_start_line;
	my $genes = $slice->get_all_Genes();
	#get ENSEMBL_ID, gene symbol, and position retaive to coding start
	while ( my $gene = shift @{$genes} ) {
	    my $gstring = ($gene->stable_id);
	    $gene_line=$gene_line."$gstring ";
	    my $gene_symbol =  $gene->external_name;
	    $gene_symbol_line = $gene_symbol_line."$gene_symbol ";
	    #get longest transcript
	    my @gene_transcripts = @{ $gene->get_all_Transcripts };
	    my $t_length = 0;
	    my $distance_to_coding_start;
	    my $strand_gene_is_on = $gene->strand;
	    foreach my $gene_transcript (@gene_transcripts) {
		$t_length_specific = $gene_transcript->length;
		if ($t_length_specific>$t_length){
			$t_length=$t_length_specific;
			if ($strand_gene_is_on eq "-1"){$distance_to_coding_start=$gene_transcript->coding_region_end;}
			if ($strand_gene_is_on eq "1"){$distance_to_coding_start=-($gene_transcript->coding_region_start);}
		}
	    }
	    $distance_to_coding_start_line = $distance_to_coding_start_line."$distance_to_coding_start ";
	}

	print outfile "$known_SNP_line\t$match_to_new_SNP\t$ESSENTIAL_SPLICE_SITE_hit\t$STOP_GAINED\t$STOP_LOST\t$FRAMESHIFT_CODING\t$NON_SYNONYMOUS_CODING\t$SPLICE_SITE\t$SYNONYMOUS_CODING\t$REGULATORY_REGION\t$FIVE_PRIME_UTR\t$THREE_PRIME_UTR\t$INTRONIC\t$UPSTREAM\t$DOWNSTREAM\t$WITHIN_NON_CODING_GENE\t$gene_line\t$gene_symbol_line\t$distance_to_coding_start_line\n";

  }
}
close(SNPfile)||die "could not close SNP file $SNP_File\n";
close(outfile)||die "could not close file $outfile_name\n";



